@@ -1,8 +1,6 @@
-
 ###
 # XML::SAX::Writer - SAX2 XML Writer
 # Robin Berjon <robin@knowscape.com>
-# 26/11/2001 - v0.01
 ###
 
 package XML::SAX::Writer::XML;
@@ -102,7 +100,7 @@ sub start_element {
     # build a string from what we have, and buffer it
     my $el = '<' . $data->{Name};
     for my $k (keys %attr_hash) {
-        $el .= ' ' . $k . '=\'' . $self->escape($attr_hash{$k}) . '\'';
+        $el .= ' ' . $k . qq[=$self->{QuoteCharecter}] . $self->escape($attr_hash{$k}) . qq[$self->{QuoteCharecter}];
     }
 
     $self->{BufferElement} = $el;
@@ -124,7 +122,7 @@ sub end_element {
     else {
         $el = '</' . $data->{Name} . '>';
     }
-    $el = $self->{Encoder}->convert($el);
+    $el = $self->safeConvert($el);
     $self->{Consumer}->output($el);
     $self->{NSHelper}->popContext;
     $self->{BufferElement} = '';
@@ -149,7 +147,7 @@ sub characters {
     else {
         $char = $self->escape($char);
     }
-    $char = $self->{Encoder}->convert($char);
+    $char = $self->safeConvert($char);
     $self->{Consumer}->output($char);
 }
 #-------------------------------------------------------------------#
@@ -182,7 +180,7 @@ sub processing_instruction {
     $self->_output_dtd;
 
     my $pi = "<?$data->{Target} $data->{Data}?>";
-    $pi = $self->{Encoder}->convert($pi);
+    $pi = $self->safeConvert($pi);
     $self->{Consumer}->output($pi);
 }
 #-------------------------------------------------------------------#
@@ -197,7 +195,7 @@ sub ignorable_whitespace {
 
     my $char = $data->{Data};
     $char = $self->escape($char);
-    $char = $self->{Encoder}->convert($char);
+    $char = $self->safeConvert($char);
     $self->{Consumer}->output($char);
 }
 #-------------------------------------------------------------------#
@@ -214,12 +212,15 @@ sub skipped_entity {
     my $ent;
     if ($data->{Name} =~ m/^%/) {
         $ent = $data->{Name} . ';';
-    }
-    else {
+
+    } elsif ($data->{Name} eq '[dtd]') {
+	# ignoring
+
+    } else {
         $ent = '&' . $data->{Name} . ';';
     }
 
-    $ent = $self->{Encoder}->convert($ent);
+    $ent = $self->safeConvert($ent);
     $self->{Consumer}->output($ent);
 
 }
@@ -246,7 +247,7 @@ sub notation_decl {
     }
     $not .= " >\n";
 
-    $not = $self->{Encoder}->convert($not);
+    $not = $self->safeConvert($not);
     $self->{Consumer}->output($not);
 }
 #-------------------------------------------------------------------#
@@ -269,8 +270,7 @@ sub unparsed_entity_decl {
     }
     $ent .= " NDATA $data->{Notation} >\n";
 
-
-    $ent = $self->{Encoder}->convert($ent);
+    $ent = $self->safeConvert($ent);
     $self->{Consumer}->output($ent);
 }
 #-------------------------------------------------------------------#
@@ -285,7 +285,8 @@ sub element_decl {
 
     # I think that param entities are normalized before this
     my $eld = "    <!ELEMENT " . $data->{Name} . ' ' . $data->{Model} . " >\n";
-    $eld = $self->{Encoder}->convert($eld);
+
+    $eld = $self->safeConvert($eld);
     $self->{Consumer}->output($eld);
 }
 #-------------------------------------------------------------------#
@@ -298,12 +299,17 @@ sub attribute_decl {
     my $data = shift;
     $self->_output_dtd;
 
+    # to be backward compatible with Perl SAX 2.0
+    $data->{Mode} = $data->{ValueDefault} 
+      if not(exists $data->{Mode}) and exists $data->{ValueDefault};
+
     # I think that param entities are normalized before this
     my $atd = "      <!ATTLIST " . $data->{eName} . ' ' . $data->{aName} . ' ';
-    $atd   .= $data->{Type} . ' ' . $data->{ValueDefault} . ' ';
+    $atd   .= $data->{Type} . ' ' . $data->{Mode} . ' ';
     $atd   .= $data->{Value} . ' ' if $data->{Value};
     $atd   .= " >\n";
-    $atd = $self->{Encoder}->convert($atd);
+
+    $atd = $self->safeConvert($atd);
     $self->{Consumer}->output($atd);
 }
 #-------------------------------------------------------------------#
@@ -318,7 +324,7 @@ sub internal_entity_decl {
 
     # I think that param entities are normalized before this
     my $ent = "    <!ENTITY " . $data->{Name} . ' \'' . $self->escape($data->{Value}) . "' >\n";
-    $ent = $self->{Encoder}->convert($ent);
+    $ent = $self->safeConvert($ent);
     $self->{Consumer}->output($ent);
 }
 #-------------------------------------------------------------------#
@@ -341,8 +347,7 @@ sub external_entity_decl {
     }
     $ent .= " >\n";
 
-
-    $ent = $self->{Encoder}->convert($ent);
+    $ent = $self->safeConvert($ent);
     $self->{Consumer}->output($ent);
 }
 #-------------------------------------------------------------------#
@@ -357,7 +362,7 @@ sub comment {
     $self->_output_dtd;
 
     my $cmt = '<!--' . $self->escapeComment($data->{Data}) . '-->';
-    $cmt = $self->{Encoder}->convert($cmt);
+    $cmt = $self->safeConvert($cmt);
     $self->{Consumer}->output($cmt);
 }
 #-------------------------------------------------------------------#
@@ -395,7 +400,7 @@ sub end_dtd {
     else {
         $dtd = ' ]>';
     }
-    $dtd = $self->{Encoder}->convert($dtd);
+    $dtd = $self->safeConvert($dtd);
     $self->{Consumer}->output($dtd);
     $self->{BufferDTD} = '';
 }
@@ -449,7 +454,7 @@ sub start_entity {
         $ent = '&' . $data->{Name} . ';';
     }
 
-    $ent = $self->{Encoder}->convert($ent);
+    $ent = $self->safeConvert($ent);
     $self->{Consumer}->output($ent);
 }
 #-------------------------------------------------------------------#
@@ -486,7 +491,7 @@ sub xml_decl {
         $xd .= '?>';
     }
 
-    $xd = $self->{Encoder}->convert($xd); # this may blow up
+    #$xd = $self->{Encoder}->convert($xd); # this may blow up
     $self->{Consumer}->output($xd);
 }
 #-------------------------------------------------------------------#
@@ -504,7 +509,7 @@ sub _output_element {
 
     if ($self->{BufferElement}) {
         my $el = $self->{BufferElement} . '>';
-        $el = $self->{Encoder}->convert($el);
+	$el = $self->safeConvert($el);
         $self->{Consumer}->output($el);
         $self->{BufferElement} = '';
     }
@@ -519,7 +524,7 @@ sub _output_dtd {
 
     if ($self->{BufferDTD}) {
         my $dtd = $self->{BufferDTD} . " [\n";
-        $dtd = $self->{Encoder}->convert($dtd);
+	$dtd = $self->safeConvert($dtd);
         $self->{Consumer}->output($dtd);
         $self->{BufferDTD} = '';
     }
@@ -551,9 +556,9 @@ Robin Berjon, robin@knowscape.com
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2002 Robin Berjon. All rights reserved. This program is
-free software; you can redistribute it and/or modify it under the same
-terms as Perl itself.
+Copyright (c) 2001-2006 Robin Berjon nad Perl XML project. All rights reserved. 
+This program is free software; you can redistribute it and/or modify it under 
+the same terms as Perl itself.
 
 =head1 SEE ALSO
 
